Security Advisories (9)
CPANSA-Mojolicious-2022-03 (2022-12-10)

Mojo::DOM did not correctly parse <script> tags.

CPANSA-Mojolicious-2021-02 (2021-06-01)

Small sessions could be used as part of a brute-force attack to decode the session secret.

CVE-2021-47208 (2021-03-16)

A bug in format detection can potentially be exploited for a DoS attack.

CVE-2018-25100 (2018-02-13)

Mojo::UserAgent::CookieJar leaks old cookies because of the missing host_only flag on empty domain.

CPANSA-Mojolicious-2015-01 (2015-02-02)

Directory traversal on Windows

CPANSA-Mojolicious-2018-03 (2018-05-19)

Mojo::UserAgent was not checking peer SSL certificates by default.

CVE-2020-36829 (2020-11-10)

Mojo::Util secure_compare can leak the string length. By immediately returning when the two strings are not the same length, the function allows an attacker to guess the length of the secret string using timing attacks.

CPANSA-Mojolicious-2018-02 (2018-05-11)

GET requests with embedded backslashes can be used to access local files on Windows hosts

CVE-2024-58134 (2025-05-03)

Mojolicious versions from 0.999922 for Perl uses a hard coded string, or the application's class name, as a HMAC session secret by default. These predictable default secrets can be exploited to forge session cookies. An attacker who knows or guesses the secret could compute valid HMAC signatures for the session cookie, allowing them to tamper with or hijack another user's session.

NAME

Mojo::IOLoop::Delay - Manage callbacks and control the flow of events

SYNOPSIS

use Mojo::IOLoop::Delay;

# Synchronize multiple events
my $delay = Mojo::IOLoop::Delay->new;
$delay->steps(sub { say 'BOOM!' });
for my $i (1 .. 10) {
  my $end = $delay->begin;
  Mojo::IOLoop->timer($i => sub {
    say 10 - $i;
    $end->();
  });
}
$delay->wait;

# Sequentialize multiple events
Mojo::IOLoop::Delay->new->steps(

  # First step (simple timer)
  sub {
    my $delay = shift;
    Mojo::IOLoop->timer(2 => $delay->begin);
    say 'Second step in 2 seconds.';
  },

  # Second step (concurrent timers)
  sub {
    my ($delay, @args) = @_;
    Mojo::IOLoop->timer(1 => $delay->begin);
    Mojo::IOLoop->timer(3 => $delay->begin);
    say 'Third step in 3 seconds.';
  },

  # Third step (the end)
  sub {
    my ($delay, @args) = @_;
    say 'And done after 5 seconds total.';
  }
)->wait;

# Handle exceptions in all steps
Mojo::IOLoop::Delay->new->steps(
  sub {
    my $delay = shift;
    die 'Intentional error';
  },
  sub {
    my ($delay, @args) = @_;
    say 'Never actually reached.';
  }
)->catch(sub {
  my ($delay, $err) = @_;
  say "Something went wrong: $err";
})->wait;

DESCRIPTION

Mojo::IOLoop::Delay manages callbacks and controls the flow of events for Mojo::IOLoop, which can help you avoid deep nested closures and memory leaks that often result from continuation-passing style.

EVENTS

Mojo::IOLoop::Delay inherits all events from Mojo::EventEmitter and can emit the following new ones.

error

$delay->on(error => sub {
  my ($delay, $err) = @_;
  ...
});

Emitted if an exception gets thrown in one of the steps, breaking the chain, fatal if unhandled.

finish

$delay->on(finish => sub {
  my ($delay, @args) = @_;
  ...
});

Emitted once the active event counter reaches zero and there are no more steps.

ATTRIBUTES

Mojo::IOLoop::Delay implements the following attributes.

ioloop

my $ioloop = $delay->ioloop;
$delay     = $delay->ioloop(Mojo::IOLoop->new);

Event loop object to control, defaults to the global Mojo::IOLoop singleton.

METHODS

Mojo::IOLoop::Delay inherits all methods from Mojo::EventEmitter and implements the following new ones.

begin

my $cb = $delay->begin;
my $cb = $delay->begin($offset);
my $cb = $delay->begin($offset, $len);

Increment active event counter, the returned callback can be used to decrement the active event counter again. Arguments passed to the callback are spliced and queued in the right order for the next step or "finish" event and "wait" method, the argument offset defaults to 1 with no default length.

# Capture all arguments except for the first one (invocant)
my $delay = Mojo::IOLoop->delay(sub {
  my ($delay, $err, $stream) = @_;
  ...
});
Mojo::IOLoop->client({port => 3000} => $delay->begin);
$delay->wait;

# Capture all arguments
my $delay = Mojo::IOLoop->delay(sub {
  my ($delay, $loop, $err, $stream) = @_;
  ...
});
Mojo::IOLoop->client({port => 3000} => $delay->begin(0));
$delay->wait;

# Capture only the second argument
my $delay = Mojo::IOLoop->delay(sub {
  my ($delay, $err) = @_;
  ...
});
Mojo::IOLoop->client({port => 3000} => $delay->begin(1, 1));
$delay->wait;

data

my $hash = $delay->data;
my $foo  = $delay->data('foo');
$delay   = $delay->data({foo => 'bar'});
$delay   = $delay->data(foo => 'bar');

Data shared between all "steps".

# Remove value
my $foo = delete $delay->data->{foo};

pass

$delay = $delay->pass;
$delay = $delay->pass(@args);

Increment active event counter and decrement it again right away to pass values to the next step.

# Longer version
$delay->begin(0)->(@args);

remaining

my $remaining = $delay->remaining;
$delay        = $delay->remaining([]);

Remaining "steps" in chain, stored outside the object to protect from circular references.

steps

$delay = $delay->steps(sub {...}, sub {...});

Sequentialize multiple events, every time the active event counter reaches zero a callback will run, the first one automatically runs during the next reactor tick unless it is delayed by incrementing the active event counter. This chain will continue until there are no more callbacks, a callback does not increment the active event counter or an exception gets thrown in a callback.

wait

$delay->wait;

Start "ioloop" and stop it again once an "error" or "finish" event gets emitted, does nothing when "ioloop" is already running.

SEE ALSO

Mojolicious, Mojolicious::Guides, http://mojolicio.us.